#!/usr/bin/python3
# -*- coding: utf-8 -*-

import time
import shutil
import tarfile
from pathlib import Path
from distutils.version import LooseVersion

from slpkg.configs import Configs
from slpkg.queries import SBoQueries
from slpkg.blacklist import Blacklist


class Utilities:

    def __init__(self):
        self.configs = Configs
        self.colors = self.configs.colour
        self.color = self.colors()
        self.yellow = self.color['yellow']
        self.cyan = self.color['cyan']
        self.endc = self.color['endc']
        self.black = Blacklist()

    def is_installed(self, name: str) -> str:
        """ Returns the installed package name. """
        installed = self.all_installed()

        for package in installed:
            pkg = self.split_installed_pkg(package)[0]

            if pkg == name:
                return package

        return ''

    def all_installed(self) -> list:
        """ Return all installed SBo packages from /val/log/packages folder. """
        installed = []
        pattern = f'*{self.configs.sbo_repo_tag}'
        var_log_packages = Path(self.configs.log_packages)

        for file in var_log_packages.glob(pattern):
            if self.split_installed_pkg(file.name) not in self.black.get():
                installed.append(file.name)

        return installed

    @staticmethod
    def untar_archive(path: str, archive: str, ext_path: str):
        """ Untar the file to the build folder. """
        tar_file = Path(path, archive)
        untar = tarfile.open(tar_file)
        untar.extractall(ext_path)
        untar.close()

    @staticmethod
    def remove_file_if_exists(path: str, file: str):
        """ Clean the old files. """
        archive = Path(path, file)
        if archive.is_file():
            archive.unlink()

    @staticmethod
    def remove_folder_if_exists(path: str, folder: str):
        """ Clean the old folders. """
        directory = Path(path, folder)
        if directory.exists():
            shutil.rmtree(directory)

    @staticmethod
    def create_folder(path: str, folder: str):
        """ Creates folder. """
        directory = Path(path, folder)
        if not directory.exists():
            directory.mkdir(parents=True, exist_ok=True)

    def split_installed_pkg(self, package: str) -> list:
        """ Split the package by the name, version, arch, build and tag. """
        name = '-'.join(package.split('-')[:-3])
        version = ''.join(package[len(name):].split('-')[:-2])
        arch = ''.join(package[len(name + version) + 2:].split('-')[:-1])
        build = ''.join(package[len(name + version + arch) + 3:].split('-')).replace(self.configs.sbo_repo_tag, '')
        tag = ''.join(package[len(name + version + arch + build) + 4:].split('-'))

        return [name, version, arch, build, tag]

    def finished_time(self, elapsed_time: float):
        """ Printing the elapsed time. """
        print(f'\n{self.yellow}Finished Successfully:{self.endc}',
              time.strftime(f'[{self.cyan}%H:%M:%S{self.endc}]',
                            time.gmtime(elapsed_time)))

    def is_repo_version_bigger(self, package: str) -> bool:
        """ Compare two versions. """
        installed = self.is_installed(package)
        if installed:
            installed_version = self.split_installed_pkg(installed)[1]
            repository_version = SBoQueries(package).version()

            return LooseVersion(repository_version) > LooseVersion(installed_version)
